package idi.diplom.linda.webservice;

import idi.diplom.linda.exceptions.DatabaseException;
import idi.diplom.linda.exceptions.GeneralException;
import idi.diplom.linda.exceptions.IllegalInputException;
import idi.diplom.linda.logic.AlarmLogic;
import idi.diplom.linda.logic.MedPacketLogic;
import idi.diplom.linda.logic.NoteLogic;
import idi.diplom.linda.logic.PatientLogic;
import idi.diplom.linda.model.Alarm;
import idi.diplom.linda.model.MedPacket;
import idi.diplom.linda.model.Note;
import idi.diplom.linda.model.Patient;

import java.util.Date;


public class LifeShirtWebService {

	private MedPacketLogic mpl = new MedPacketLogic();
	private PatientLogic pl = new PatientLogic();
	private NoteLogic nl = new NoteLogic();
	private AlarmLogic al = new AlarmLogic();
	
	/**
	 * Gets the last saved medpacket on specified patient
	 * @param patient to get monitoring data from
	 * @return MedPacket medpacket
	 * @throws DatabaseException
	 * @throws IllegalInputException
	 * @throws GeneralException
	 */
	public MedPacket getLastMedPacket(Patient p) throws DatabaseException, IllegalInputException, GeneralException{
		if(p==null){
			throw new IllegalInputException("Patient was null");
		}
		MedPacket mp = mpl.getLast(p);
		return mp;
	}
	
	/**
	 * Saves a new patient to the database
	 * @param patient to be saved
	 * @throws DatabaseException
	 * @throws IllegalInputException
	 * @throws GeneralException
	 */
	public void saveNewPatient(Patient p) throws DatabaseException, IllegalInputException, GeneralException{
		System.out.println("savenewpatient called");
		if(p==null){
			throw new IllegalInputException("Patient was null");
		}
		pl.savePatient(p);
	}
	
	/**
	 * Gets all patients saved in database
	 * @return a list of all patients
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	public Patient[] getAllPatients() throws DatabaseException, GeneralException{
		Patient[] pArray = pl.getAllPatients();
		return pArray;
	}

	/**
	 * Updates a patient's profile
	 * @param patient to be updated
	 * @throws DatabaseException
	 * @throws IllegalInputException
	 * @throws GeneralException
	 */
	public void updatePatient(Patient p) throws DatabaseException, IllegalInputException, GeneralException{
		if(p==null){
			throw new IllegalInputException("Patient was null");
		}
		pl.savePatient(p);
	}
	
	/**
	 * Saves a note to the database
	 * @param note to be saved
	 * @throws DatabaseException
	 * @throws IllegalInputException
	 * @throws GeneralException
	 */
	public void saveNote(Note n) throws DatabaseException, IllegalInputException, GeneralException{
		if(n==null){
			throw new IllegalInputException("Note was null");
		}
		nl.saveNote(n);
	}
	
	/**
	 * Receives all notes saved on specified patient
	 * @param patient to get notes on
	 * @return a list of all notes
	 * @throws DatabaseException
	 * @throws IllegalInputException
	 * @throws GeneralException
	 */
	public Note[] getAllNotes(Patient p) throws  DatabaseException, IllegalInputException, GeneralException{
		if (p == null)
			throw new IllegalInputException("Patient was null");
		return nl.getAllNotes(p);
	}

	/**
	 * Dismissed the specified alarm
	 * @param alarm to be dismissed
	 * @throws DatabaseException
	 * @throws IllegalInputException
	 * @throws GeneralException
	 */
	public void dismissAlarm(Alarm a) throws DatabaseException, IllegalInputException, GeneralException{
		if(a==null){
			System.out.println("a er null");
			throw new IllegalInputException("Alarm was null");
		}
		al.dismissAlarm(a);
	}
	
	/**
	 * Gets all alarms saved on specified patient that are not yet dismissed
	 * @param patient to get alarms on
	 * @return a list of all active alarms
	 * @throws DatabaseException
	 * @throws IllegalInputException
	 * @throws GeneralException
	 */
	public Alarm[] getAllActiveAlarms(Patient p) throws DatabaseException, IllegalInputException, GeneralException{
		Alarm[] aArray = al.getAllActiveAlarms(p);
		//replaces all medpacket arrays to null since the client won't need them at this point
		if(aArray!=null){
			for(int i=0; i<aArray.length; i++){
				Alarm a = aArray[i];
				a.setMedPackets(null);
			}
		}
		return aArray;
	}

	/**
	 * Returns a list of medpackets saved on specified patient between specified time.
	 * Every resolution packet is returned.
	 * @param patient to get monitoring data from
	 * @param fromTime
	 * @param toTime
	 * @param a number telling what packets should be returned
	 * @return an array of medpackets
	 * @throws DatabaseException
	 * @throws IllegalInputException
	 * @throws GeneralException
	 */
	public MedPacket[] getPacketsBetweenDates(Patient p, Date fromTime, Date toTime, int resolution) throws DatabaseException, IllegalInputException, GeneralException{
		if(p==null || fromTime==null || toTime==null){
			throw new IllegalInputException("Parts of input was null");
		}
		MedPacket[] mArray = mpl.getMedPacketsBetweenDates(p, fromTime, toTime,  resolution);
		return mArray;
	}
	

	
	
	
}
